/****************************** Module Header *******************************
* Module Name: snoop.h
*
* Main header file for snoop.
* 
****************************************************************************/

#include <windows.h>
#include <commctrl.h>		// for status bar
	
#define FASTDELTA   8		// the jump distance using Shift+'Arrow Key'

// clip x such that it is in the range between min and max
#define BOUND(x,min,max) ((x) < (min) ? (min) : ((x) > (max) ? (max) : (x)))

#define MM10PERINCH 254                     // Tenths of a millimeter per inch.

#define NPAL        256                     // Number of palette entries.


// -------------------------------- resource id definitions ---------------------------
// menu items
#define MENU_HELP_ABOUT					100

#define MENU_EDIT_COPY					200
#define MENU_EDIT_REFRESH				201
#define MENU_EDIT_SAVE_CONFIG			202
#define MENU_EDIT_AUTO_SAVE_CONFIG		203
#define MENU_EDIT_QUIT					204

#define MENU_OPTIONS_REFRESHRATE		300
#define MENU_OPTIONS_HISTOGRAM			301
#define MENU_OPTIONS_ONTOP				302
#define MENU_GRID_ENABLED				303
#define MENU_RULER_ENABLED				304
#define MENU_CENTER_HIGHLIGHT_ENABLED	305

#define MENU_SCREEN						306
#define MENU_ACTIVE_WINDOW				307
#define MENU_INVERSE_Y					308
#define MENU_DISPLAY_HEX				309
#define MENU_BOUND						310

#define	MENU_MOUSE_MODE					330
#define	MENU_ARROW_MODE					331
#define	MENU_TRACK_MODE					332

#define MENU_GRAPH_OFF					401
#define	MENU_GRAPH_HOR_STEP				402
#define	MENU_GRAPH_HOR_LINEAR			403
#define	MENU_GRAPH_VER_STEP				404
#define	MENU_GRAPH_VER_LINEAR			405
#define	MENU_GRAPH_HIGHLIGHT			406

#define MENU_GRAPH_STATISTICS			407
#define MENU_STATISTICS_INFO			408

#define MENU_FILTER_SNOOP				500
#define MENU_FILTER_FIRST				MENU_FILTER_SNOOP
#define MENU_FILTER_GRADIENT			501
#define MENU_FILTER_LAST				MENU_FILTER_GRADIENT

// for dialog boxes
#define DID_ABOUT						1000

#define DID_REFRESHRATE					1100
#define DID_REFRESHRATEENABLE			1101
#define DID_REFRESHRATEINTERVAL			1102

#define DID_HISTOGRAM					2000

#define DID_HISTOGRAM_R					2001
#define DID_HISTOGRAM_G					2002
#define DID_HISTOGRAM_B					2003

#define DID_HISTOGRAM_AUTO_NOR			2004
#define DID_GRADIENT_AUTO_NOR			2005

#define DID_HISTOGRAM_REF				2006
#define DID_GRADIENT_REF				2007

#define DID_SLIDER_BRIGHTNESS			2100
#define DID_SLIDER_CONTRAST				2101
#define DID_SLIDER_GAMMA				2102

#define DID_EDIT_BRIGHTNESS				2103
#define DID_EDIT_CONTRAST				2104
#define DID_EDIT_GAMMA					2105

#define DID_UPDATE						2200
#define DID_RESET						2201

// other resources
#define IDMENU_SNOOP					2000

#define IDACCEL_SNOOP					3000

#define IDTIMER_SNOOP					4000


// some constant definitions
#define SB_HEIGHT						18		// height of the status bar 
												// must improve later
#define INSET_BORDER					6

#define RULER_HEIGHT					12	

#define HISTOGRAM_RULER_HEIGHT			8

#define TEXTBOX_HEIGHT					15		// all these number found emprically
#define TEXTBOX_HEIGHT_2_ROWS			30
#define TEXTBOX_HEIGHT_4_ROWS			60
#define TEXTBOX_WIDTH_2_COLS			272
#define TEXTBOX_WIDTH_1_COLS			135

// number of configurations
#define NUM_CONFIG	10
			
// ------------------------------ function prototypes ---------------------------------

BOOL InitInstance(HINSTANCE hInst, INT cmdShow);
HPALETTE CreatePhysicalPalette(VOID);
LONG APIENTRY AppWndProc(HWND hwnd, UINT msg, WPARAM wParam, LPARAM lParam);
VOID CalcZoomedSize(VOID);
VOID CalcRGBGraphSize(VOID);
VOID DoTheGrid(HDC hdc, HGDIOBJ hPen);
VOID DoTheZoomIn(HDC hdc);
VOID MoveView(INT nDirectionCode, BOOL fFast, BOOL fPeg);
VOID ResizeWindow(INT nDirectionCode, BOOL fFast);
VOID DrawZoomRect(VOID);
VOID EnableRefresh(BOOL fEnable);
VOID CopyToClipboard(VOID);
BOOL APIENTRY AboutDlgProc(HWND hwnd, UINT msg, WPARAM wParam, LPARAM lParam);
BOOL APIENTRY RefreshRateDlgProc(HWND hwnd, UINT msg, WPARAM wParam, LPARAM lParam);
BOOL APIENTRY HistogramDlgProc(HWND hwnd, UINT msg, WPARAM wParam, LPARAM lParam);
VOID ResetTitle();
VOID DoTheRGBGraph(HDC hdc, HDC hdcScreen);
VOID DoTheStatistics(HDC hdc, HDC hdcScreen);
VOID DoTheGradientGraph(HDC hdc, HDC hdcScreen);
VOID DrawTheGradientStatistics(HDC hdc, HDC hdcScreen);
void DoTheRuler(HDC hdc);
void DoTheGradient(HDC hdc, HDC hdcScreen);

VOID FillinColorBuffer(HDC hdcScreen);
VOID DoTheHighlight(HDC hdc, HDC hdcScreen);

void ProcessCommandLine(char *gszCmdLine, HWND hwnd, HMENU hmenu);

int DetectColorDepth();

VOID InitScreenCapture();
VOID ScreenCapture(HDC hdcScreen, int x, int y, int width, int height, 
				   unsigned char **buf, int *padding);
VOID ReleaseScreenCapture();

void GetWindowsDimension(int *width, int *height);

int findIndex(int zoom);

// Status bar functions
HWND InitStatusBar(HWND hwndParent);

// in registryIO.cpp
VOID SaveAutoSaveRegistry();
VOID SaveRegistry(HWND hwnd, int n=1);
VOID LoadRegistry(HWND hwnd, HMENU hmenu, int n=1);
VOID RemoveRegistry();
